@@ -1,14 +1,11 @@
-import sys
-
 from ploomber.cli.parsers import CustomParser
 from ploomber.cli.io import cli_endpoint
 from ploomber.telemetry import telemetry
-import datetime
 
 
 @cli_endpoint
+@telemetry.log_call('report')
 def main():
-    start_time = datetime.datetime.now()
     parser = CustomParser(description='Make a pipeline report',
                           prog='ploomber report')
     with parser:
@@ -17,11 +14,8 @@ def main():
             '-o',
             help='Where to save the report, defaults to pipeline.html',
             default='pipeline.html')
+
     dag, args = parser.load_from_entry_point_arg()
     dag.to_markup(path=args.output)
-    end_time = datetime.datetime.now()
-    telemetry.log_api("ploomber_report",
-                      total_runtime=str(end_time - start_time),
-                      dag=dag,
-                      metadata={'argv': sys.argv})
+
     print('Report saved at:', args.output)
