@@ -216,6 +216,13 @@ def run_finished(headers, runid):
     return response
 
 
+@auth_header
+def run_latest_failed(headers, reason):
+    if reason != "none":
+        _requests.get(f"{HOST}/runs/latest/failed", headers=headers)
+        click.echo('Marked latest run as failed...')
+
+
 @auth_header
 def products_list(headers):
     res = _requests.get(f"{HOST}/products", headers=headers).json()
@@ -317,6 +324,8 @@ def upload_project(force=False,
                    github_owner=None,
                    github_repo=None,
                    verbose=False):
+    # TODO: this should use the function in the default.py module to load
+    # the default entry-point
     dag = DAGSpec('pipeline.yaml').to_dag().render(show_progress=False)
 
     # TODO: test
