@@ -1,10 +1,11 @@
 import os
 import shlex
 import pdb
+import shutil
 import tempfile
 import subprocess
 from subprocess import PIPE
-from pathlib import Path
+from pathlib import Path, PurePosixPath
 from importlib.util import find_spec
 import warnings
 
@@ -111,6 +112,13 @@ def _check_exporter(exporter, path_to_output):
                 'and try again')
 
 
+def _safe_suffix(product):
+    try:
+        return Path(product).suffix
+    except Exception:
+        return None
+
+
 class NotebookConverter:
     """
     Thin wrapper around nbconvert to provide a simple API to convert .ipynb
@@ -176,6 +184,7 @@ class NotebookConverter:
         self._nbconvert_export_kwargs = nbconvert_export_kwargs or {}
 
     def convert(self):
+
         if self._exporter is None and self._nbconvert_export_kwargs:
             warnings.warn(
                 f'Output {self._path_to_output!r} is a '
@@ -452,19 +461,24 @@ class NotebookRunner(NotebookMixin, Task):
         information to choose a kernel or the notebook already includes
         kernelspec data (in metadata.kernelspec), this is ignored, otherwise,
         the kernel is looked up using jupyter_client.kernelspec.get_kernel_spec
-    nbconvert_exporter_name: str, optional
+    nbconvert_exporter_name: str or dict, optional
         Once the notebook is run, this parameter controls whether to export
         the notebook to a different parameter using the nbconvert package,
         it is not needed unless the extension cannot be used to infer the
         final output format, in which case the nbconvert.get_exporter is used.
+        If nb_product_key is a list of multiple nb products keys,
+        nbconvert_exporter_name should be a dict containing keys from this
+        list.
     ext_in: str, optional
         Source extension. Required if loading from a str. If source is a
         ``pathlib.Path``, the extension from the file is used.
-    nb_product_key: str, optional
+    nb_product_key: str or list, optional
         If the notebook is expected to generate other products, pass the key
         to identify the output notebook (i.e. if product is a list with 3
         ploomber.File, pass the index pointing to the notebook path). If the
         only output is the notebook itself, this parameter is not needed
+        If multiple notebook conversions are required like html, pdf, this
+        parameter should be a list of keys like 'nb_ipynb', 'nb_html, 'nb_pdf'.
     static_analysis : ('disabled', 'regular', 'strict'), default='regular'
         Check for various errors in the notebook. In 'regular' mode, it aborts
         execution if the notebook has syntax issues, or similar problems that
@@ -484,7 +498,6 @@ class NotebookRunner(NotebookMixin, Task):
         Defaults to False. This resembles the default behavior when
         running notebooks interactively via `jupyter notebook`
 
-
     Examples
     --------
 
@@ -512,6 +525,34 @@ class NotebookRunner(NotebookMixin, Task):
                 # must be generated by nb.ipynb
                 data: data.csv
 
+    Spec API (multiple notebook products, added in ``0.19.6``):
+
+    (generate the executed notebooks in multiple formats)
+
+    .. code-block:: yaml
+        :class: text-editor
+        :name: pipeline-yaml
+
+        tasks:
+          - source: script.py
+            # keys can be named as per user's choice. None
+            # of the keys are mandatory. However, every key mentioned
+            # in this list should be a part of the product dict below.
+            nb_product_key: [nb_ipynb, nb_pdf, nb_html]
+            # When nb_product_key is a list, nbconvert_exporter_name
+            # should be a dict with required keys from nb_product_key
+            # only. If missing, it uses the default exporter
+            nbconvert_exporter_name:
+                nb_pdf: webpdf
+            # Every notebook product defined here should correspond to key
+            # defined in nb_product_key.
+            product:
+                nb_ipynb: nb.ipynb
+                nb_pdf: doc.pdf
+                nb_html: report.html
+                # must be generated by nb.ipynb
+                data: data.csv
+
     Python API:
 
     >>> from pathlib import Path
@@ -548,11 +589,62 @@ class NotebookRunner(NotebookMixin, Task):
 
     Notes
     -----
-    nbconvert's documentation:
-    https://nbconvert.readthedocs.io/en/latest/config_options.html#preprocessor-options
+    .. collapse:: changelog
+
+        .. versionchanged:: 0.19.6
+            Support for generating output notebooks in multiple formats, see
+            example above
+
+    `nbconvert's documentation <https://nbconvert.readthedocs.io/en/latest/config_options.html#preprocessor-options>`_ # noqa
     """
     PRODUCT_CLASSES_ALLOWED = (File, )
 
+    def _validate_nbconvert_exporter(self):
+        if isinstance(self.nb_product_key, list) and isinstance(
+                self.nbconvert_exporter_name, str):
+            raise TaskInitializationError(
+                f"When specifying nb_product_key as a list, "
+                f"create a dictionary under nbconvert_exporter_name "
+                f"with required keys in {self.nb_product_key!r}. ")
+
+        if isinstance(self.nb_product_key, str) and isinstance(
+                self.nbconvert_exporter_name, dict):
+            raise TaskInitializationError(
+                "Please specify a single nbconvert_exporter_name "
+                "when generating a single notebook product. "
+                "Example : 'nbconvert_exporter_name': '<exporter name'> ")
+
+        if isinstance(self.nb_product_key, list) and isinstance(
+                self.nbconvert_exporter_name, dict):
+            for exporter_name in self.nbconvert_exporter_name:
+                if exporter_name not in self.nb_product_key:
+                    raise TaskInitializationError(
+                        f"Invalid nbconvert exporter : {exporter_name!r}. "
+                        f"All exporter names in nbconvert_exporter_name "
+                        f"should be present in nb_product_key")
+
+    def _validate_nb_product_key(self):
+        for key in self.nb_product_key:
+            if self.product.get(key) is None:
+                raise TaskInitializationError(
+                    f"Missing key '{key}' in "
+                    f"product: {(str(self.product))!r}. "
+                    f"All keys specified in "
+                    f"{self.nb_product_key!r} must contain the "
+                    f"corresponding product path.")
+
+        for key, path in self.product.to_json_serializable().items():
+            if _safe_suffix(
+                    path
+            ) in ['.md', '.html', '.tex', '.pdf', '.rst', '.ipynb'] \
+                    and key not in self.nb_product_key:
+                raise TaskInitializationError(
+                    f"Missing key '{key}' in "
+                    f"nb_product_key: {self.nb_product_key!r}. "
+                    f"All notebook product keys specified in "
+                    f"{(str(self.product))!r} must be specified "
+                    f"in nb_product_key as well.")
+
     @requires(['jupyter', 'papermill', 'jupytext'], 'NotebookRunner')
     def __init__(self,
                  source,
@@ -590,24 +682,47 @@ class NotebookRunner(NotebookMixin, Task):
                                                    check_if_kernel_installed)
         super().__init__(product, dag, name, params)
 
+        self._validate_nbconvert_exporter()
+
         if isinstance(self.product, MetaProduct):
-            if self.product.get(nb_product_key) is None:
+            if isinstance(self.nb_product_key, list):
+                self._validate_nb_product_key()
+
+            elif self.product.get(nb_product_key) is None:
                 raise TaskInitializationError(
                     f"Missing key '{nb_product_key}' in "
                     f"product: {(str(self.product))!r}. "
                     f"{nb_product_key!r} must contain "
                     "the path to the output notebook.")
 
-        if isinstance(self.product, MetaProduct):
-            product_nb = (
-                self.product[self.nb_product_key]._identifier.best_repr(
-                    shorten=False))
-        else:
-            product_nb = self.product._identifier.best_repr(shorten=False)
+        if isinstance(self.nb_product_key, str):
+            if isinstance(self.product, MetaProduct):
+                product_nb = (
+                    self.product[self.nb_product_key]._identifier.best_repr(
+                        shorten=False))
 
-        self._converter = NotebookConverter(product_nb,
-                                            nbconvert_exporter_name,
-                                            nbconvert_export_kwargs)
+            else:
+                product_nb = self.product._identifier.best_repr(shorten=False)
+
+        else:
+            multiple_product_nb = {
+                key: (self.product[key]._identifier.best_repr(shorten=False))
+                for key in self.nb_product_key
+            }
+
+        if isinstance(self.nb_product_key, str):
+            self._converter = NotebookConverter(product_nb,
+                                                nbconvert_exporter_name,
+                                                nbconvert_export_kwargs)
+        else:
+            self._converter = []
+            for key, product_nb in multiple_product_nb.items():
+
+                exporter = nbconvert_exporter_name.get(
+                    key) if nbconvert_exporter_name else None
+                self._converter.append(
+                    NotebookConverter(product_nb, exporter,
+                                      nbconvert_export_kwargs))
 
     @staticmethod
     def _init_source(source,
@@ -630,14 +745,38 @@ class NotebookRunner(NotebookMixin, Task):
         if self.static_analysis == 'regular':
             self.source._check_notebook(raise_=True, check_signature=False)
 
+        multiple_nb_products = []
+
+        # the task generates more than one product
         if isinstance(self.product, MetaProduct):
-            path_to_out = Path(str(self.product[self.nb_product_key]))
+
+            # generating a single output notebook
+            if isinstance(self.nb_product_key, str):
+                nb_product = self.product[self.nb_product_key]
+
+            # generates multiple output notebooks
+            else:
+                # grab the last one by default
+                key_to_use = self.nb_product_key[-1]
+
+                for key in self.nb_product_key:
+                    path = Path(str(self.product[key]))
+                    multiple_nb_products.append(path)
+
+                    # but prefer the one which is already an ipynb file
+                    if path.suffix == '.ipynb':
+                        key_to_use = key
+
+                nb_product = self.product[key_to_use]
+
+        # only one product - it must be the output notebook
         else:
-            path_to_out = Path(str(self.product))
+            nb_product = self.product
 
         # we will run the notebook with this extension, regardless of the
         # user's choice, if any error happens, this will allow them to debug
         # we will change the extension after the notebook runs successfully
+        path_to_out = Path(str(nb_product))
         path_to_out_ipynb = path_to_out.with_suffix('.ipynb')
 
         fd, tmp = tempfile.mkstemp('.ipynb')
@@ -657,11 +796,23 @@ class NotebookRunner(NotebookMixin, Task):
             pm.execute_notebook(str(tmp), str(path_to_out_ipynb),
                                 **self.papermill_params)
         except Exception as e:
-            raise TaskBuildError(
-                'Error when executing task'
-                f' {self.name!r}. Partially'
-                f' executed notebook available at {str(path_to_out_ipynb)}'
-            ) from e
+            # upload partially executed notebook if there's a clint
+            if nb_product.client:
+                nb_product.client.upload(str(path_to_out_ipynb))
+                parent = nb_product.client.parent
+                remote_path = str(PurePosixPath(parent, path_to_out_ipynb))
+
+                raise TaskBuildError('Error when executing task'
+                                     f' {self.name!r}. Partially'
+                                     ' executed notebook uploaded to '
+                                     'remote storage at: '
+                                     f'{remote_path}') from e
+            else:
+                raise TaskBuildError(
+                    'Error when executing task'
+                    f' {self.name!r}. Partially'
+                    f' executed notebook available at {str(path_to_out_ipynb)}'
+                ) from e
         finally:
             tmp.unlink()
 
@@ -669,8 +820,17 @@ class NotebookRunner(NotebookMixin, Task):
         if path_to_out_ipynb != path_to_out and path_to_out.is_file():
             path_to_out.unlink()
 
+        for product in multiple_nb_products:
+            if path_to_out_ipynb != product:
+                shutil.copyfile(path_to_out_ipynb, product)
+
         path_to_out_ipynb.rename(path_to_out)
-        self._converter.convert()
+
+        if isinstance(self._converter, list):
+            for _converter in self._converter:
+                _converter.convert()
+        else:
+            self._converter.convert()
 
 
 class ScriptRunner(NotebookMixin, Task):
